@@ -1,7 +1,12 @@
 # -*- coding: utf-8 -*-
 
 """
-传输的数据包格式:data\r\n
+tcp:
+    传输的数据包格式:raw_data\r\n
+udp:
+    传输的数据包格式:raw_data
+    因为并不通过末尾的符号做识别
+
 解析之后的json:
 {
     endpoint: 'get_name'        str
@@ -20,6 +25,7 @@ import json
 import asyncore
 import asynchat
 import socket
+import constants
 
 logger = logging.getLogger('gourd')
 
@@ -62,6 +68,7 @@ class TcpServer(asyncore.dispatcher, RouteMgr):
     _host = None
     _port = None
     _backlog = None
+    _terminator = None
 
     def __init__(self):
         # 把map变成私有的
@@ -71,7 +78,7 @@ class TcpServer(asyncore.dispatcher, RouteMgr):
     def handle_accept(self):
         sock, address = self.accept()
         logger.debug('accept. socket:%s, address:%s', sock, address)
-        TcpServerHandler(self, sock, self._map)
+        TcpRequestHandler(self, address, self._terminator, sock, self._map)
 
     def _init_socket(self):
         self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -79,26 +86,29 @@ class TcpServer(asyncore.dispatcher, RouteMgr):
         self.bind((self._host, self._port))
         self.listen(self._backlog)
 
-    def run(self, host, port, backlog):
+    def run(self, host, port, backlog=None, terminator=None):
         self._host = host
         self._port = port
-        self._backlog = backlog
+        self._backlog = backlog or constants.DEFAULT_BACKLOG
+        self._terminator = terminator or constants.DEFAULT_TERMINATOR
 
         self._init_socket()
 
         asyncore.loop(map=self._map)
 
 
-class TcpServerHandler(asynchat.async_chat):
+class TcpRequestHandler(asynchat.async_chat):
     _server = None
+    _address = None
     _raw_data = None
     _json_data = None
 
-    def __init__(self, server, sock, map=None):
+    def __init__(self, server, address, terminator, sock, map=None):
         asynchat.async_chat.__init__(self, sock, map)
         self._server = server
+        self._address = address
         # 用json做格式，结尾加上个 \r\n 就可以了。
-        self.set_terminator('\r\n')
+        self.set_terminator(terminator)
 
     def collect_incoming_data(self, data):
         # 直接用默认提供的方法就行
@@ -139,10 +149,6 @@ class TcpServerHandler(asynchat.async_chat):
         asynchat.async_chat.handle_close(self)
         logger.debug('socket closed')
 
-    @property
-    def raw_data(self):
-        return self._raw_data
-
     @property
     def server(self):
         """
@@ -150,9 +156,102 @@ class TcpServerHandler(asynchat.async_chat):
         """
         return self._server
 
+    @property
+    def address(self):
+        return self._address
+
+    @property
+    def raw_data(self):
+        return self._raw_data
+
     @property
     def json_data(self):
         """
         获取获取的json数据
         """
         return self._json_data
+
+
+class UdpServer(asyncore.dispatcher, RouteMgr):
+    _address = None
+    _raw_data = None
+    _json_data = None
+
+    _host = None
+    _port = None
+
+    def __init__(self):
+        # 把map变成私有的
+        asyncore.dispatcher.__init__(self, map=dict())
+        RouteMgr.__init__(self)
+
+    def handle_read(self):
+        self._raw_data, self._address = self.socket.recvfrom(constants.DEFAULT_PACKAGE_SIZE)
+        try:
+            self._json_data = json.loads(self._raw_data)
+        except Exception, e:
+            logger.error('json parse fail. e: %s, recv_data: %s', e, self._raw_data)
+            return
+
+        logger.debug('jdata: %s', self._json_data)
+
+        if not isinstance(self._json_data, dict):
+            logger.error('json_data is not dict. json_data: %s', self._json_data)
+            return
+
+        if not self._json_data.get('endpoint'):
+            logger.error('endpoint is not in json_data. json_data: %s', self._json_data)
+            return
+
+        endpoint = self._json_data['endpoint']
+        view_func = self.server.get_view_func(endpoint)
+        if not view_func:
+            logger.error('endpoint is not valid. endpoint: %s', endpoint)
+            return
+
+        try:
+            view_func(self)
+        except Exception, e:
+            logger.error('view_func raise exception. endpoint: %s, view_func: %s, e: %s',
+                         endpoint, view_func.__name__, e)
+
+    def _init_socket(self):
+        self.create_socket(socket.AF_INET, socket.SOCK_DGRAM)
+        self.bind((self._host, self._port))
+
+    def run(self, host, port):
+        self._host = host
+        self._port = port
+
+        self._init_socket()
+
+        asyncore.loop(map=self._map)
+
+    def push(self, data):
+        """
+        发送数据，为了tcp一致
+        """
+        return self.socket.sendto(data, self._address)
+
+    @property
+    def server(self):
+        """
+        获取当前server
+        """
+        return self
+
+    @property
+    def address(self):
+        return self._address
+
+    @property
+    def raw_data(self):
+        return self._raw_data
+
+    @property
+    def json_data(self):
+        """
+        获取获取的json数据
+        """
+        return self._json_data
+
